"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/qp/__init__.py $
$Id: __init__.py 30570 2008-03-11 14:44:15Z dbinger $
Copyright (c) Corporation for National Research Initiatives 2008. All Rights Reserved.
"""
__version__ = "2.1"